from django.test import TestCase, Client
from django.contrib.auth import get_user_model

User = get_user_model()


class BaseTestCase(TestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.user_email = 'user@email.com'
        self.user_password = 'the_password'
        self.user = User.objects.create_user(
            username='testuser', email=self.user_email, 
            password=self.user_password)

        self.auth_client = Client()
        self.auth_client.force_login(self.user)


class SuperuserBaseTestCase(BaseTestCase):

    def setUp(self):
        self.superuser_email = 'superuser@email.com'
        self.superuser_password = 'the_password'
        self.superuser = User.objects.create_superuser(
            username='testsuperuser',
            email=self.superuser_email,
            password=self.superuser_password)

        self.super_client = Client()
        self.super_client.force_login(self.superuser)

        super(SuperuserBaseTestCase, self).setUp()


class TestRequiresAuthenticatedUser(object):

    def test_requires_authenticated_user(self):
        response = self.client.get(self.view_url)
        self.assertEqual(response.status_code, 302)


class TestRequiresSuperuser(TestRequiresAuthenticatedUser):

    def test_requires_superuser(self):
        response = self.auth_client.get(self.view_url)
        self.assertEqual(response.status_code, 302)

    def test_superuser_get_request_success(self):
        response = self.super_client.get(self.view_url)
        self.assertEqual(response.status_code, 200)


from celery.beat import Service
from django.utils import timezone
import datetime
import json


def get_periodic_tasks_info():
    from celery import current_app
    schedule = Service(current_app).get_scheduler().get_schedule()
    tasks = []
    for key, entry in schedule.items():
        # entry.is_due() returns (is_due, time_in_seconds_for_next_execution)
        is_due_tpl = entry.is_due()

        next_execution = timezone.now() + datetime.timedelta(seconds=is_due_tpl[1])

        # remove delay between the timezone.now and the schedule entry due date
        next_execution = next_execution.replace(microsecond=0)

        tasks.append({
            'name': key,
            'task': entry.task,
            'args': '(' + ', '.join([json.dumps(arg) for arg in entry.args]) + ')',
            'kwargs': json.dumps(entry.kwargs),
            'is_due': is_due_tpl[0],
            'next_execution': next_execution
        })

    return tasks



from django.utils import version as django_version

from celerybeat_status.tests.utils import (
    SuperuserBaseTestCase, TestRequiresSuperuser)

if django_version.get_complete_version() < (2, 0, 0):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class PeriodicTaskStatusListTests(
        TestRequiresSuperuser, SuperuserBaseTestCase):
    view_name = 'periodic-tasks-status'

    def setUp(self):
        super(PeriodicTaskStatusListTests, self).setUp()
        self.view_url = reverse(self.view_name)



from django.views.generic import TemplateView
from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from celerybeat_status.helpers import get_periodic_tasks_info


class PeriodicTasksStatusListView(TemplateView):
    template_name = "celerybeat_status/periodic_tasks_status_list.html"
    site_url = "/"

    @method_decorator(user_passes_test(
        lambda u: u.is_staff and u.is_superuser,
        login_url='admin:login'
    ))
    def dispatch(self, request, *args, **kwargs):
        return super(PeriodicTasksStatusListView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PeriodicTasksStatusListView, self).get_context_data(**kwargs)
        context['title'] = _('Periodic tasks status')
        context['user'] = self.request.user
        context['site_url'] = self.site_url
        context['has_permission'] = self.request.user.is_superuser
        context['tasks'] = get_periodic_tasks_info()

        return context



# -*- coding: utf-8
__title__ = 'CeleryBeat Status'
__version__ = '0.0.10'
__author__ = 'Vinta Software'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2017 Vinta Serviços e Soluções Tecnológicas Ltda'

# Version synonym
VERSION = __version__



from django.conf.urls import url

from celerybeat_status.views import PeriodicTasksStatusListView


urlpatterns = [
    url(r'^periodic-tasks/$', PeriodicTasksStatusListView.as_view(),
        name='periodic-tasks-status'),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import shutil
import sys
from io import open

from setuptools import setup

long_description = ''
try:
    from pypandoc import convert
    long_description = convert('README.md', 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version('celerybeat_status')


if sys.argv[-1] == 'publish':
    try:
        import pypandoc
    except ImportError:
        print("pypandoc not installed.\nUse `pip install pypandoc`.\nExiting.")
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    shutil.rmtree('dist')
    shutil.rmtree('build')
    shutil.rmtree('django_celerybeat_status.egg-info')
    sys.exit()


setup(
    name='django-celerybeat-status',
    version=version,
    license='MIT',
    description='A simple django admin extension that shows when your periodic are going to run next',
    long_description=long_description,
    author='Vinta Software',
    author_email='contact@vinta.com.br',
    packages=get_packages('celerybeat_status'),
    package_data=get_package_data('celerybeat_status'),
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.6',
    ]
)
